import axios, { AxiosInstance, AxiosRequestConfig, AxiosResponse } from 'axios';
import { getToken, removeToken } from './auth';

// 定义后端返回的统一数据结构
interface ApiResponse<T = any> {
  code: number;
  data: T;
  msg: string;
}

// 创建 Axios 实例
const createAxiosInstance = (): AxiosInstance => {
  const instance = axios.create({
    baseURL: import.meta.env.VITE_API_BASE_URL as string, // 从 Vite 环境变量获取 baseURL
    timeout: 10000, // 请求超时时间
    headers: {
      'Content-Type': 'application/json',
    },
  });

  // 请求拦截器
  instance.interceptors.request.use(
    (config) => {
      // 添加 token 等请求头
      const token = getToken()
      if (token) {
        config.headers.Authorization = `Bearer ${token}`;
      }
      return config;
    },
    (error) => {
      return Promise.reject(error);
    }
  );

  // 响应拦截器
  instance.interceptors.response.use(
    (response: AxiosResponse<ApiResponse>) => {
      const { code, msg } = response.data;

      // 401 未授权，跳转首页
      if (code === 401) {
        // 这里调用退出登录的逻辑
        console.log('未授权，跳转首页');
        // 清除用户信息
        removeToken();
        // 跳转首页
        window.location.href = '/'; //后续用发布订阅模式修改
        return Promise.reject(new Error(msg || '未授权'));
      }

      // 其他非 200 状态码
      if (code !== 200) {
        console.error(`请求错误 ${code}: ${msg}`);
        // 这里可以根据需要弹出错误提示
        return Promise.reject(new Error(msg || '请求错误'));
      }

      // 返回数据部分
      return response.data.data;
    },
    (error) => {
      // 处理 HTTP 状态码不是 200 的情况
      if (error.response) {
        const { status, data } = error.response;
        console.error(`HTTP 错误 ${status}:`, data);
      } else {
        console.error('请求错误:', error.message);
      }
      return Promise.reject(error);
    }
  );

  return instance;
};

const http = createAxiosInstance();

// 封装通用的 request 函数
async function request<T = any>(config: AxiosRequestConfig): Promise<T> {
  try {
    const { data } = await http.request<ApiResponse<T>>(config);
    return data.data;
  } catch (error) {
    // 这里可以统一处理错误，或者让调用者自己处理
    throw error;
  }
}

// 封装常用的 HTTP 方法
const httpClient = {
  get<T = any>(url: string, config?: AxiosRequestConfig): Promise<T> {
    return request<T>({ ...config, method: 'GET', url });
  },

  post<T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<T> {
    return request<T>({ ...config, method: 'POST', url, data });
  },

  put<T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<T> {
    return request<T>({ ...config, method: 'PUT', url, data });
  },

  delete<T = any>(url: string, config?: AxiosRequestConfig): Promise<T> {
    return request<T>({ ...config, method: 'DELETE', url });
  },

  patch<T = any>(url: string, data?: any, config?: AxiosRequestConfig): Promise<T> {
    return request<T>({ ...config, method: 'PATCH', url, data });
  },
};

export default httpClient;